#include <stdio.h>
#include "config.h"

/* The lang is filled by reading the configuration file */

const char * BCITES_VERSION = B_VR;

int bcites_debug=0;

typedef struct book BOOK;

struct book
     {
     char * abbrev; /* the abbreviation of the book */
     BOOK * next; /* the next abbreviation (where more than one 
				abbreviations are used) */
     };

typedef struct books BOOKS;

struct books
     {
     BOOK * book; /* the list of abbreviations of this book */
     BOOKS * next; /* then next book */
     };

typedef struct lang LANG;
struct lang 
     {
     char * name; /* the name of the language <== '{'*/
  
     char chars[CHS_CNT]; /* set of chars used in biblical cites <== '!' */
     char * lower; /* set of lower chars (the alfabet) <= '_' */
     char * upper; /* set of upper char ( dtto ) <='^' */
     char * parts; /* chars for signing the parts of verses <= '&' */
     BOOKS * books; /* names of biblical books */
     LANG * next; /* then next language struct */
     };

LANG * config_table;

 /* a routine (simillar to fgets) to read lines from file,
  but output is the last char (usefull for testing if the EOF occured) */ 

int readcr(char * strin, int delka ,FILE * fd) {
int c,i;
i=0;c=0;
for ( ;((c=getc(fd)) != EOF) && (i<(delka--)) && (c!='\n'); )
 if ((c!='\n') && (c!=13)) { strin[i]=c; i++; };
 
  strin[i]='\0';
return c;
 }


/* counts chars in a string to the next space or '%' */

size_t strl2sp(char * s){
size_t i;
i=0;
while ((s[i]!='\0') && (s[i]!=' ') && (s[i]!='%'))  i++;
return i;
}

/* allocates a string and fills it with appropriate value */

char * newst (char * s)
{
char * bla;
if ((bla=(char *)malloc(strl2sp(s)))==NULL)
 {
 printf("Init (bcites) : can't allocate space for a string !\n");
 exit (1);
 }
memcpy(bla,s,strl2sp(s));
return bla;
} 

char * newstt (char * s)
{
char * bla;
if ((bla=(char *)malloc(strlen(s)+1))==NULL)
 {
 printf("Init (bcites) : can't allocate space for a string !\n");
 exit (1);
 }
memcpy(bla,s,strlen(s)+1);
return bla;
} 


void cst (int lng) {
if ((config_table==NULL) || (lng!=1)) 
 {
 fprintf(stderr,"Init (bcites) : config tag not inside '{ .. }' !\n");
 exit (1);
 }
}

 /* ------------ init ----------------- */

void bcites_init() {
FILE * conff;
LANG * nxt=NULL;
BOOK * nxb=NULL;
BOOK * tb;
BOOKS * nxbs=NULL;
BOOKS * tbs;
char buffer[BUF_SIZE+1];
int lng=0;
int i;

config_table=NULL;
if ((conff=fopen(INIFILE,"r"))==NULL)
   {
   fprintf(stderr,"Init (bcites) : can't open config file !\n");
   exit (1);
   }   

while (readcr(buffer, BUF_SIZE, conff)!=EOF)
 {
 switch(buffer[0])
  {
  case '%': break; /* Just comment */
 
  case '}': if (lng!=1) /* closing language definition */
    {
    fprintf(stderr,"Init (bcites) : no '{' to close !\n");
    exit (1);
    };
   lng=0; break;

  case '{': if (lng>0) /* opening language difinition */
    {
    fprintf(stderr,"Init (bcites) : languages can't be nested !\n");
    exit (1);
    };
   if ((buffer[1]=='\0')  || (buffer[1]==' '))
    {
    fprintf(stderr,"Init (bcites) : no language specified (or leading space) !\n");
    exit (1);
    }
   /* allocate the LANG table for the current language */ 
   if ((config_table=(LANG *)malloc(sizeof(LANG)))==NULL)
    {
    fprintf(stderr,"Init (bcites) : can't malloc !\n");
    exit(1);
    }
   config_table->next=nxt;
   nxt=config_table;
   config_table->name=newst(buffer+1);  /* the name of this language */
   config_table->books=NULL;   /* no book-list yet */
   lng=1;
   break;
   
  case '_': cst(lng); config_table->lower=newst(buffer+1); break;
  case '^': cst(lng); config_table->upper=newst(buffer+1); break;
  case '&': cst(lng); config_table->parts=newst(buffer+1); break;

  case '!': cst(lng); 
  for (i=1; i<=CHS_CNT; i++) {
  if ((buffer[i]=='\0') || (buffer[i]==' '))
   {
   fprintf(stderr,"Init (bcites) : invalid number of chars (???) !\n");
   exit(1);
   }
  config_table->chars[i-1]=buffer[i];
  }; 
  break;

  case '+': cst(lng);
   if ((nxbs=(BOOKS *)malloc(sizeof(BOOKS)))==NULL)
    {
    fprintf(stderr,"Init (bcites) : can't malloc !\n");
    exit (1);
    }
   if (config_table->books==NULL) 
     config_table->books=nxbs;
    else   
     tbs->next=nxbs;
   tbs=nxbs;
   if ((tb=(BOOK *)malloc(sizeof(BOOK)))==NULL)
    {
    fprintf(stderr,"Init (bcites) : can't malloc!\n");
    exit (1);
    }
    tb->abbrev=newstt(buffer+1);
    tbs->book=tb;
    tb->next=NULL;
    nxb=tb;
    break;
    
  case '*': cst(lng);
   if ((nxb=(BOOK *)malloc(sizeof(BOOK)))==NULL)
    {
    fprintf(stderr,"Init (bcites) : can't malloc!\n");
    exit (1);
    }
    nxb->abbrev=newstt(buffer+1);
    tb->next=nxb;
    tb=nxb;
    break;
  }  
 } 

fclose(conff);
}  /* ------ end of 'init' ----------------- */

LANG * findlang(char * s) {

LANG * curr;
LANG * found;

curr=config_table;
found=NULL;
while(curr!=NULL) 
 {
 if (!strcmp(s,curr->name)) found=curr;
 curr=curr->next;
 }
return found;
}   /* ---------------- */

char * booknr(LANG * lg, int bok)
{
BOOKS * curbs;
int curn;

if (bok<1) return UNKNOWN_BOOK;
else 
{
curbs=lg->books;
curn=1;
while ((curn < bok) && (curbs!=NULL))
{
 curn++;
 curbs=curbs->next;
}
if ((curn<bok) || (curbs==NULL)) 
 {
 fprintf(stderr,"booknr (bcites): invalid book number (%d>%d)! \n",bok,curn);
 exit (1);
 }
return curbs->book->abbrev;
} /* endif */ 
} /* ----------------- */


/* prints a book-abbreviation with it's variants */

void bookprn(FILE * f,LANG * lg, int bok)
{
BOOKS * curbs;
int curn;
BOOK * curb;
int stat=0;

if (bok<1) fprintf(f,UNKNOWN_BOOK);
else
{
curbs=lg->books;
curn=1;
while ((curn < bok) && (curbs!=NULL))
{
 curn++;
 curbs=curbs->next;
}
if ((curn<bok) || (curbs==NULL)) 
 {
 fprintf(stderr,"bookprn (bcites): invalid book number ! \n");
 exit (1);
 }

curb=curbs->book;
if (curb->abbrev==NULL)
 {
 fprintf(stderr,"bookprn (bcites) : non-consistency of the data structure !\n");
 exit (1);
 }
fprintf(f,"%s",curb->abbrev);
while (curb->next!=NULL)
 {
 curb=curb->next;
 if (curb->abbrev==NULL)
  {
  fprintf(stderr,"bookprn (bcites) : non-consistency of the data structure !\n");
  exit (1);
  }
 if (stat==0) fprintf(f," ("); 
 else printf(", "); 
 fprintf(f,"%s",curb->abbrev);
 stat=1;
 }
if (stat==1) fprintf(f,")");
}  
} /* ----------------- */

    
